@@ -7,7 +7,8 @@
 
 #include <coreplugin/icore.h>
 
-#include <cppeditor/cppcodestylepreferencesfactory.h>
+#include <cppeditor/cppqtstyleindenter.h>
+#include <cppeditor/cpptoolssettings.h>
 
 #include <extensionsystem/pluginmanager.h>
 #include <extensionsystem/pluginspec.h>
@@ -30,13 +31,7 @@ namespace ClangFormat {
 
 static bool isBeautifierPluginActivated()
 {
-    const QVector<ExtensionSystem::PluginSpec *> specs = ExtensionSystem::PluginManager::plugins();
-    return std::find_if(specs.begin(),
-                        specs.end(),
-                        [](ExtensionSystem::PluginSpec *spec) {
-                            return spec->name() == "Beautifier" && spec->isEffectivelyEnabled();
-                        })
-           != specs.end();
+    return ExtensionSystem::PluginManager::specExistsAndIsEnabled("beautifier");
 }
 
 static bool isBeautifierOnSaveActivated()
@@ -61,24 +56,29 @@ ClangFormatIndenter::ClangFormatIndenter(QTextDocument *doc)
 
 bool ClangFormatIndenter::formatCodeInsteadOfIndent() const
 {
+#ifdef WITH_TESTS
+    if (CppEditor::CppToolsSettings::cppCodeStyle()->codeStyleSettings().forceFormatting)
+        return true;
+#endif
     return getCurrentIndentationOrFormattingSettings(m_fileName)
            == ClangFormatSettings::Mode::Formatting;
 }
 
 std::optional<TabSettings> ClangFormatIndenter::tabSettings() const
 {
+    // FIXME: Why don't we have a valid file path from the beginning?
+    if (m_fileName.isEmpty())
+        return {};
+
     FormatStyle style = styleForFile();
     TabSettings tabSettings;
 
     switch (style.UseTab) {
-    case FormatStyle::UT_Never:
-        tabSettings.m_tabPolicy = TabSettings::SpacesOnlyTabPolicy;
-        break;
     case FormatStyle::UT_Always:
         tabSettings.m_tabPolicy = TabSettings::TabsOnlyTabPolicy;
         break;
     default:
-        tabSettings.m_tabPolicy = TabSettings::MixedTabPolicy;
+        tabSettings.m_tabPolicy = TabSettings::SpacesOnlyTabPolicy;
     }
 
     tabSettings.m_tabSize = static_cast<int>(style.TabWidth);
@@ -116,7 +116,7 @@ bool ClangFormatIndenter::formatWhileTyping() const
 ClangFormatForwardingIndenter::ClangFormatForwardingIndenter(QTextDocument *doc)
     : TextEditor::Indenter(doc)
     , m_clangFormatIndenter(std::make_unique<ClangFormatIndenter>(doc))
-    , m_cppIndenter(CppEditor::CppCodeStylePreferencesFactory().createIndenter(doc))
+    , m_cppIndenter(CppEditor::createCppQtStyleIndenter(doc))
 {}
 
 ClangFormatForwardingIndenter::~ClangFormatForwardingIndenter() = default;
@@ -188,7 +188,7 @@ bool ClangFormatForwardingIndenter::formatOnSave() const
 }
 
 TextEditor::IndentationForBlock ClangFormatForwardingIndenter::indentationForBlocks(
-    const QVector<QTextBlock> &blocks,
+    const QList<QTextBlock> &blocks,
     const TextEditor::TabSettings &tabSettings,
     int cursorPositionInEditor)
 {
@@ -228,4 +228,9 @@ std::optional<int> ClangFormatForwardingIndenter::margin() const
     return currentIndenter()->margin();
 }
 
+bool ClangFormatForwardingIndenter::respectsTabSettings() const
+{
+    return currentIndenter()->respectsTabSettings();
+}
+
 } // namespace ClangFormat
