@@ -1,97 +1,77 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "projectnodes.h"
 
+#include "buildsystem.h"
 #include "project.h"
 #include "projectexplorerconstants.h"
-#include "projecttree.h"
+#include "target.h"
 
-#include <coreplugin/fileiconprovider.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/iversioncontrol.h>
 #include <coreplugin/vcsmanager.h>
 
-#include <utils/algorithm.h>
 #include <utils/fileutils.h>
+#include <utils/fsengine/fileiconprovider.h>
 #include <utils/hostosinfo.h>
-#include <utils/mimetypes/mimedatabase.h>
-#include <utils/mimetypes/mimetype.h>
+#include <utils/mimeconstants.h>
+#include <utils/mimeutils.h>
 #include <utils/pointeralgorithm.h>
 #include <utils/qtcassert.h>
+#include <utils/threadutils.h>
+#include <utils/utilsicons.h>
 
-#include <QFileInfo>
 #include <QDir>
+#include <QFileInfo>
 #include <QIcon>
-#include <QStyle>
-#include <QTimer>
 
 #include <memory>
 
+using namespace Utils;
+
 namespace ProjectExplorer {
 
-static FolderNode *folderNode(const FolderNode *folder, const Utils::FilePath &directory)
-{
-    return static_cast<FolderNode *>(Utils::findOrDefault(folder->folderNodes(),
-                                                          [&directory](const FolderNode *fn) {
-        return fn && fn->filePath() == directory;
-    }));
-}
+QHash<QString, QIcon> DirectoryIcon::m_cache;
 
 static FolderNode *recursiveFindOrCreateFolderNode(FolderNode *folder,
-                                                   const Utils::FilePath &directory,
-                                                   const Utils::FilePath &overrideBaseDir,
+                                                   const FilePath &directory,
+                                                   const FilePath &overrideBaseDir,
                                                    const FolderNode::FolderNodeFactory &factory)
 {
     Utils::FilePath path = overrideBaseDir.isEmpty() ? folder->filePath() : overrideBaseDir;
 
-    Utils::FilePath directoryWithoutPrefix;
+    QString directoryWithoutPrefix;
     bool isRelative = false;
 
-    if (path.isEmpty() || path.toFileInfo().isRoot()) {
-        directoryWithoutPrefix = directory;
+    if (path.isEmpty() || path.isRootPath()) {
+        directoryWithoutPrefix = directory.path();
         isRelative = false;
     } else {
         if (directory.isChildOf(path) || directory == path) {
             isRelative = true;
-            directoryWithoutPrefix = directory.relativeChildPath(path);
+            directoryWithoutPrefix = directory.relativeChildPath(path).path();
         } else {
-            isRelative = false;
-            path.clear();
-            directoryWithoutPrefix = directory;
+            const QString relativePath = directory.relativePathFromDir(path);
+            if (relativePath.count("../") < 5) {
+                isRelative = true;
+                directoryWithoutPrefix = relativePath;
+            } else {
+                isRelative = false;
+                path.clear();
+                directoryWithoutPrefix = directory.path();
+            }
         }
     }
-    QStringList parts = directoryWithoutPrefix.toString().split('/', QString::SkipEmptyParts);
-    if (!Utils::HostOsInfo::isWindowsHost() && !isRelative && parts.count() > 0)
+    QStringList parts = directoryWithoutPrefix.split('/', Qt::SkipEmptyParts);
+    if (directory.osType() != OsTypeWindows && !isRelative && !parts.isEmpty())
         parts[0].prepend('/');
 
     ProjectExplorer::FolderNode *parent = folder;
-    foreach (const QString &part, parts) {
-        path = path.pathAppended(part);
+    for (const QString &part : std::as_const(parts)) {
+        path = path.pathAppended(part).cleanPath();
         // Find folder in subFolders
-        FolderNode *next = folderNode(parent, path);
+        FolderNode *next = parent->folderNode(path);
         if (!next) {
             // No FolderNode yet, so create it
             auto tmp = factory(path);
@@ -152,11 +132,8 @@ void Node::setIsGenerated(bool g)
         m_flags = static_cast<NodeFlag>(m_flags & ~FlagIsGenerated);
 }
 
-void Node::setAbsoluteFilePathAndLine(const Utils::FilePath &path, int line)
+void Node::setAbsoluteFilePathAndLine(const FilePath &path, int line)
 {
-    if (m_filePath == path && m_line == line)
-        return;
-
     m_filePath = path;
     m_line = line;
 }
@@ -212,6 +189,15 @@ const ProjectNode *Node::managingProject() const
     return const_cast<Node *>(this)->managingProject();
 }
 
+Project *Node::getProject() const
+{
+    if (const ContainerNode * const cn = asContainerNode())
+        return cn->project();
+    if (!m_parentFolderNode)
+        return nullptr;
+    return m_parentFolderNode->getProject();
+}
+
 /*!
   The path of the file or folder in the filesystem the node represents.
   */
@@ -243,6 +229,65 @@ bool Node::isEnabled() const
     return parent ? parent->isEnabled() : true;
 }
 
+QIcon FileNode::icon() const
+{
+    if (hasError())
+        return Utils::Icons::WARNING.icon();
+    if (m_icon.isNull())
+        m_icon = Utils::FileIconProvider::icon(filePath());
+    return m_icon;
+}
+
+void FileNode::setIcon(const QIcon icon)
+{
+    m_icon = icon;
+}
+
+bool FileNode::hasError() const
+{
+    return m_hasError;
+}
+
+void FileNode::setHasError(bool error)
+{
+    m_hasError = error;
+}
+
+void FileNode::setHasError(bool error) const
+{
+    m_hasError = error;
+}
+
+Core::IVersionControl::FileState FileNode::modificationState() const
+{
+    if (isGenerated())
+        return Core::IVersionControl::FileState::Unknown;
+
+    if (!m_modificationState) {
+        const FilePath dir = filePath().absolutePath();
+        if (Core::IVersionControl *vc = Core::VcsManager::findVersionControlForDirectory(dir))
+            m_modificationState = vc->modificationState(filePath());
+        else
+            m_modificationState = Core::IVersionControl::FileState::Unknown;
+    }
+    return *m_modificationState;
+}
+
+bool FileNode::useUnavailableMarker() const
+{
+    return m_useUnavailableMarker;
+}
+
+void FileNode::setUseUnavailableMarker(bool useUnavailableMarker)
+{
+    m_useUnavailableMarker = useUnavailableMarker;
+}
+
+void FileNode::resetModificationState()
+{
+    m_modificationState.reset();
+}
+
 /*!
   Returns \c true if the file is automatically generated by a compile step.
   */
@@ -274,22 +319,21 @@ void Node::setParentFolderNode(FolderNode *parentFolder)
     m_parentFolderNode = parentFolder;
 }
 
-FileType Node::fileTypeForMimeType(const Utils::MimeType &mt)
+FileType Node::fileTypeForMimeType(const MimeType &mt)
 {
+    using namespace Utils::Constants;
     FileType type = FileType::Source;
     if (mt.isValid()) {
         const QString mtName = mt.name();
-        if (mtName == Constants::C_HEADER_MIMETYPE
-                || mtName == Constants::CPP_HEADER_MIMETYPE)
+        if (mtName == C_HEADER_MIMETYPE || mtName == CPP_HEADER_MIMETYPE)
             type = FileType::Header;
-        else if (mtName == Constants::FORM_MIMETYPE)
+        else if (mtName == FORM_MIMETYPE)
             type = FileType::Form;
-        else if (mtName == Constants::RESOURCE_MIMETYPE)
+        else if (mtName == RESOURCE_MIMETYPE)
             type = FileType::Resource;
-        else if (mtName == Constants::SCXML_MIMETYPE)
+        else if (mtName == SCXML_MIMETYPE)
             type = FileType::StateChart;
-        else if (mtName == Constants::QML_MIMETYPE
-                 || mtName == Constants::QMLUI_MIMETYPE)
+        else if (mtName == QML_MIMETYPE || mtName == QMLUI_MIMETYPE)
             type = FileType::QML;
     } else {
         type = FileType::Unknown;
@@ -299,8 +343,55 @@ FileType Node::fileTypeForMimeType(const Utils::MimeType &mt)
 
 FileType Node::fileTypeForFileName(const Utils::FilePath &file)
 {
-    return fileTypeForMimeType(Utils::mimeTypeForFile(file.toString(),
-                                                      Utils::MimeMatchMode::MatchExtension));
+    return fileTypeForMimeType(Utils::mimeTypeForFile(file, Utils::MimeMatchMode::MatchExtension));
+}
+
+FilePath Node::pathOrDirectory(bool dir) const
+{
+    const FolderNode *folder = asFolderNode();
+    if (isVirtualFolderType() && folder) {
+        FilePath location;
+        // Virtual Folder case
+        // If there are files directly below or no subfolders, take the folder path
+        auto Any = [](auto) { return true; };
+        if (folder->findChildFileNode(Any) || !folder->findChildFolderNode(Any)) {
+            location = m_filePath;
+        } else {
+            // Otherwise we figure out a commonPath from the subfolders
+            FilePaths list;
+            folder->forEachFolderNode([&](FolderNode *f) { list << f->filePath(); });
+            location = list.commonPath();
+        }
+
+        QFileInfo fi = location.toFileInfo();
+        while ((!fi.exists() || !fi.isDir()) && !fi.isRoot() && (fi.fileName() != fi.absolutePath()))
+            fi.setFile(fi.absolutePath());
+        return FilePath::fromString(fi.absoluteFilePath());
+    }
+
+    if (m_filePath.isEmpty())
+        return {};
+
+    if (!m_filePath.isLocal()) {
+        if (dir)
+            return m_filePath.isDir() ? m_filePath.absoluteFilePath() : m_filePath.absolutePath();
+        return m_filePath;
+    }
+
+    FilePath location;
+    QFileInfo fi = m_filePath.toFileInfo();
+    // remove any /suffixes, which e.g. ResourceNode uses
+    // Note this could be removed again by making path() a true path again
+    // That requires changes in both the VirtualFolderNode and ResourceNode
+    while (!fi.exists() && !fi.isRoot())
+        fi.setFile(fi.absolutePath());
+
+    if (dir)
+        location = FilePath::fromString(fi.isDir() ? fi.absoluteFilePath() : fi.absolutePath());
+    else
+        location = FilePath::fromString(fi.absoluteFilePath());
+
+    return location;
 }
 
 /*!
@@ -317,6 +408,9 @@ FileNode::FileNode(const Utils::FilePath &filePath, const FileType fileType) :
     m_fileType(fileType)
 {
     setFilePath(filePath);
+    const bool ignored = (fileType == FileType::Project || fileType == FileType::App
+                          || fileType == FileType::Lib);
+    setUseUnavailableMarker(!ignored && filePath.isLocal() && !filePath.exists());
     setListInProject(true);
     if (fileType == FileType::Project)
         setPriority(DefaultProjectFilePriority);
@@ -340,65 +434,6 @@ FileType FileNode::fileType() const
     return m_fileType;
 }
 
-static QList<FileNode *> scanForFilesRecursively(const Utils::FilePath &directory,
-                                                 const std::function<FileNode *(const Utils::FilePath &)> factory,
-                                                 QSet<QString> &visited, QFutureInterface<QList<FileNode*>> *future,
-                                                 double progressStart, double progressRange,
-                                                 const QList<Core::IVersionControl*> &versionControls)
-{
-    QList<FileNode *> result;
-
-    const QDir baseDir = QDir(directory.toString());
-
-    // Do not follow directory loops:
-    const int visitedCount = visited.count();
-    visited.insert(baseDir.canonicalPath());
-    if (visitedCount == visited.count())
-        return result;
-
-    const QFileInfoList entries = baseDir.entryInfoList(QStringList(), QDir::AllEntries|QDir::NoDotAndDotDot);
-    double progress = 0;
-    const double progressIncrement = progressRange / static_cast<double>(entries.count());
-    int lastIntProgress = 0;
-    for (const QFileInfo &entry : entries) {
-        if (future && future->isCanceled())
-            return result;
-
-        const Utils::FilePath entryName = Utils::FilePath::fromString(entry.absoluteFilePath());
-        if (!Utils::contains(versionControls, [&entryName](const Core::IVersionControl *vc) {
-                             return vc->isVcsFileOrDirectory(entryName);
-            })) {
-            if (entry.isDir())
-                result.append(scanForFilesRecursively(entryName, factory, visited, future, progress, progressIncrement, versionControls));
-            else if (FileNode *node = factory(entryName))
-                result.append(node);
-        }
-        if (future) {
-            progress += progressIncrement;
-            const int intProgress = std::min(static_cast<int>(progressStart + progress), future->progressMaximum());
-            if (lastIntProgress < intProgress) {
-                future->setProgressValue(intProgress);
-                lastIntProgress = intProgress;
-            }
-        }
-    }
-    if (future)
-        future->setProgressValue(std::min(static_cast<int>(progressStart + progressRange), future->progressMaximum()));
-    return result;
-}
-
-QList<FileNode *>
-FileNode::scanForFiles(const Utils::FilePath &directory,
-                       const std::function<FileNode *(const Utils::FilePath &)> factory,
-                       QFutureInterface<QList<FileNode *>> *future)
-{
-    QSet<QString> visited;
-    if (future)
-        future->setProgressRange(0, 1000000);
-    return scanForFilesRecursively(directory, factory, visited, future, 0.0, 1000000.0,
-                                   Core::VcsManager::versionControls());
-}
-
 bool FileNode::supportsAction(ProjectAction action, const Node *node) const
 {
     if (action == InheritedFromParent)
@@ -442,16 +477,28 @@ QString FolderNode::displayName() const
 }
 
 /*!
-  Contains the icon that should be used in a view. Default is the directory icon
- (QStyle::S_PDirIcon).
-  s\a setIcon()
+    Contains the icon that should be used in a view. Default is the directory icon
+    (QStyle::S_PDirIcon). Calling this method is only safe in the UI thread.
+
+    \sa setIcon()
  */
 QIcon FolderNode::icon() const
 {
+    QTC_CHECK(isMainThread());
+
     // Instantiating the Icon provider is expensive.
-    if (m_icon.isNull())
-        m_icon = Core::FileIconProvider::icon(QFileIconProvider::Folder);
-    return m_icon;
+    if (auto strPtr = std::get_if<QString>(&m_icon)) {
+        m_icon = QIcon(*strPtr);
+    } else if (auto directoryIconPtr = std::get_if<DirectoryIcon>(&m_icon)) {
+        m_icon = directoryIconPtr->icon();
+    } else if (auto creatorPtr = std::get_if<IconCreator>(&m_icon)) {
+        m_icon = (*creatorPtr)();
+    } else {
+        auto iconPtr = std::get_if<QIcon>(&m_icon);
+        if (!iconPtr || iconPtr->isNull())
+            m_icon = Utils::FileIconProvider::icon(QFileIconProvider::Folder);
+    }
+    return std::get<QIcon>(m_icon);
 }
 
 Node *FolderNode::findNode(const std::function<bool(Node *)> &filter)
@@ -528,6 +575,22 @@ void FolderNode::forEachProjectNode(const std::function<void(const ProjectNode *
     }
 }
 
+void FolderNode::forEachFileNode(const std::function<void (FileNode *)> &fileTask) const
+{
+    for (const std::unique_ptr<Node> &n : m_nodes) {
+        if (FileNode *fn = n->asFileNode())
+            fileTask(fn);
+    }
+}
+
+void FolderNode::forEachFolderNode(const std::function<void (FolderNode *)> &folderTask) const
+{
+    for (const std::unique_ptr<Node> &n : m_nodes) {
+        if (FolderNode *fn = n->asFolderNode())
+            folderTask(fn);
+    }
+}
+
 ProjectNode *FolderNode::findProjectNode(const std::function<bool(const ProjectNode *)> &predicate)
 {
     if (ProjectNode *projectNode = asProjectNode()) {
@@ -544,21 +607,31 @@ ProjectNode *FolderNode::findProjectNode(const std::function<bool(const ProjectN
     return nullptr;
 }
 
+FolderNode *FolderNode::findChildFolderNode(const std::function<bool(FolderNode *)> &predicate) const
+{
+    for (const std::unique_ptr<Node> &n : m_nodes) {
+        if (FolderNode *fn = n->asFolderNode())
+            if (predicate(fn))
+                return fn;
+    }
+    return nullptr;
+}
+
+FileNode *FolderNode::findChildFileNode(const std::function<bool(FileNode *)> &predicate) const
+{
+    for (const std::unique_ptr<Node> &n : m_nodes) {
+        if (FileNode *fn = n->asFileNode())
+            if (predicate(fn))
+                return fn;
+    }
+    return nullptr;
+}
+
 const QList<Node *> FolderNode::nodes() const
 {
     return Utils::toRawPointer<QList>(m_nodes);
 }
 
-QList<FileNode*> FolderNode::fileNodes() const
-{
-    QList<FileNode *> result;
-    for (const std::unique_ptr<Node> &n : m_nodes) {
-        if (FileNode *fn = n->asFileNode())
-            result.append(fn);
-    }
-    return result;
-}
-
 FileNode *FolderNode::fileNode(const Utils::FilePath &file) const
 {
     return static_cast<FileNode *>(Utils::findOrDefault(m_nodes,
@@ -568,14 +641,13 @@ FileNode *FolderNode::fileNode(const Utils::FilePath &file) const
     }));
 }
 
-QList<FolderNode*> FolderNode::folderNodes() const
+FolderNode *FolderNode::folderNode(const Utils::FilePath &directory) const
 {
-    QList<FolderNode *> result;
-    for (const std::unique_ptr<Node> &n : m_nodes) {
-        if (FolderNode *fn = n->asFolderNode())
-            result.append(fn);
-    }
-    return result;
+    Node *node = Utils::findOrDefault(m_nodes, [directory](const std::unique_ptr<Node> &n) {
+        FolderNode *fn = n->asFolderNode();
+        return fn && fn->filePath() == directory;
+    });
+    return static_cast<FolderNode *>(node);
 }
 
 void FolderNode::addNestedNode(std::unique_ptr<FileNode> &&fileNode,
@@ -591,8 +663,28 @@ void FolderNode::addNestedNodes(std::vector<std::unique_ptr<FileNode> > &&files,
                                 const Utils::FilePath &overrideBaseDir,
                                 const FolderNode::FolderNodeFactory &factory)
 {
-    for (std::unique_ptr<FileNode> &f : files)
-        addNestedNode(std::move(f), overrideBaseDir, factory);
+    using DirWithNodes = std::pair<Utils::FilePath, std::vector<std::unique_ptr<FileNode>>>;
+    std::vector<DirWithNodes> fileNodesPerDir;
+    for (auto &f : files) {
+        const Utils::FilePath parentDir = f->filePath().parentDir();
+        const auto it = std::lower_bound(fileNodesPerDir.begin(), fileNodesPerDir.end(), parentDir,
+            [](const DirWithNodes &nad, const Utils::FilePath &dir) { return nad.first < dir; });
+        if (it != fileNodesPerDir.end() && it->first == parentDir) {
+            it->second.emplace_back(std::move(f));
+        } else {
+            DirWithNodes dirWithNodes;
+            dirWithNodes.first = parentDir;
+            dirWithNodes.second.emplace_back(std::move(f));
+            fileNodesPerDir.insert(it, std::move(dirWithNodes));
+        }
+    }
+
+    for (DirWithNodes &dirWithNodes : fileNodesPerDir) {
+        FolderNode * const folderNode = recursiveFindOrCreateFolderNode(this, dirWithNodes.first,
+                                                                        overrideBaseDir, factory);
+        for (auto &f : dirWithNodes.second)
+            folderNode->addNode(std::move(f));
+    }
 }
 
 // "Compress" a tree of foldernodes such that foldernodes with exactly one foldernode as a child
@@ -601,29 +693,31 @@ void FolderNode::addNestedNodes(std::vector<std::unique_ptr<FileNode> > &&files,
 // files.
 void FolderNode::compress()
 {
-    if (auto subFolder = m_nodes.size() == 1 ? m_nodes.at(0)->asFolderNode() : nullptr) {
-        const bool sameType = (isFolderNodeType() && subFolder->isFolderNodeType())
-                || (isProjectNodeType() && subFolder->isProjectNodeType())
-                || (isVirtualFolderType() && subFolder->isVirtualFolderType());
-        if (!sameType)
-            return;
+    // Child nodes need to be compressed first.
+    forEachFolderNode([&](FolderNode *fn) { fn->compress(); });
 
-        // Only one subfolder: Compress!
-        setDisplayName(QDir::toNativeSeparators(displayName() + "/" + subFolder->displayName()));
-        for (Node *n : subFolder->nodes()) {
-            std::unique_ptr<Node> toMove = subFolder->takeNode(n);
-            toMove->setParentFolderNode(nullptr);
-            addNode(std::move(toMove));
-        }
-        setAbsoluteFilePathAndLine(subFolder->filePath(), -1);
+    // There must be exactly one child node, which has to be of the same type as this node.
+    if (m_nodes.size() != 1)
+        return;
+    const auto subFolder = m_nodes.front()->asFolderNode();
+    if (!subFolder)
+        return;
+    const bool sameType = (isFolderNodeType() && subFolder->isFolderNodeType())
+                          || (isProjectNodeType() && subFolder->isProjectNodeType())
+                          || (isVirtualFolderType() && subFolder->isVirtualFolderType());
+    if (!sameType)
+        return;
 
-        takeNode(subFolder);
-
-        compress();
-    } else {
-        for (FolderNode *fn : folderNodes())
-            fn->compress();
+    // Now do the compression by moving the child node's children into this node
+    // and removing the child node.
+    for (Node *n : subFolder->nodes()) {
+        std::unique_ptr<Node> toMove = subFolder->takeNode(n);
+        toMove->setParentFolderNode(nullptr);
+        addNode(std::move(toMove));
     }
+    setDisplayName(QDir::toNativeSeparators(displayName() + "/" + subFolder->displayName()));
+    setAbsoluteFilePathAndLine(subFolder->filePath(), -1);
+    takeNode(subFolder);
 }
 
 bool FolderNode::replaceSubtree(Node *oldNode, std::unique_ptr<Node> &&newNode)
@@ -656,14 +750,41 @@ void FolderNode::setDisplayName(const QString &name)
     m_displayName = name;
 }
 
+/*!
+    Sets the \a icon for this node. Note that creating QIcon instances is only safe in the UI thread.
+*/
 void FolderNode::setIcon(const QIcon &icon)
 {
     m_icon = icon;
 }
 
+/*!
+    Sets the \a directoryIcon that is used to create the icon for this node on demand.
+*/
+void FolderNode::setIcon(const DirectoryIcon &directoryIcon)
+{
+    m_icon = directoryIcon;
+}
+
+/*!
+    Sets the \a path that is used to create the icon for this node on demand.
+*/
+void FolderNode::setIcon(const QString &path)
+{
+    m_icon = path;
+}
+
+/*!
+    Sets the \a iconCreator function that is used to create the icon for this node on demand.
+*/
+void FolderNode::setIcon(const IconCreator &iconCreator)
+{
+    m_icon = iconCreator;
+}
+
 void FolderNode::setLocationInfo(const QList<FolderNode::LocationInfo> &info)
 {
-    m_locations = info;
+    m_locations = Utils::sorted(info, &LocationInfo::priority);
 }
 
 const QList<FolderNode::LocationInfo> FolderNode::locationInfo() const
@@ -688,7 +809,7 @@ bool FolderNode::supportsAction(ProjectAction action, const Node *node) const
     return parentFolder && parentFolder->supportsAction(action, node);
 }
 
-bool FolderNode::addFiles(const QStringList &filePaths, QStringList *notAdded)
+bool FolderNode::addFiles(const FilePaths &filePaths, FilePaths *notAdded)
 {
     ProjectNode *pn = managingProject();
     if (pn)
@@ -696,15 +817,14 @@ bool FolderNode::addFiles(const QStringList &filePaths, QStringList *notAdded)
     return false;
 }
 
-bool FolderNode::removeFiles(const QStringList &filePaths, QStringList *notRemoved)
+RemovedFilesFromProject FolderNode::removeFiles(const FilePaths &filePaths, FilePaths *notRemoved)
 {
-    ProjectNode *pn = managingProject();
-    if (pn)
+    if (ProjectNode * const pn = managingProject())
         return pn->removeFiles(filePaths, notRemoved);
-    return false;
+    return RemovedFilesFromProject::Error;
 }
 
-bool FolderNode::deleteFiles(const QStringList &filePaths)
+bool FolderNode::deleteFiles(const FilePaths &filePaths)
 {
     ProjectNode *pn = managingProject();
     if (pn)
@@ -712,25 +832,32 @@ bool FolderNode::deleteFiles(const QStringList &filePaths)
     return false;
 }
 
-bool FolderNode::canRenameFile(const QString &filePath, const QString &newFilePath)
+bool FolderNode::canRenameFile(const FilePath &oldFilePath, const FilePath &newFilePath)
 {
     ProjectNode *pn = managingProject();
     if (pn)
-        return pn->canRenameFile(filePath, newFilePath);
+        return pn->canRenameFile(oldFilePath, newFilePath);
     return false;
 }
 
-bool FolderNode::renameFile(const QString &filePath, const QString &newFilePath)
+bool FolderNode::renameFiles(const FilePairs &filesToRename, FilePaths *notRenamed)
 {
     ProjectNode *pn = managingProject();
     if (pn)
-        return pn->renameFile(filePath, newFilePath);
+        return pn->renameFiles(filesToRename, notRenamed);
     return false;
 }
 
-FolderNode::AddNewInformation FolderNode::addNewInformation(const QStringList &files, Node *context) const
+bool FolderNode::addDependencies(const QStringList &dependencies)
 {
-    Q_UNUSED(files);
+    if (ProjectNode * const pn = managingProject())
+        return pn->addDependencies(dependencies);
+    return false;
+}
+
+FolderNode::AddNewInformation FolderNode::addNewInformation(const FilePaths &files, Node *context) const
+{
+    Q_UNUSED(files)
     return AddNewInformation(displayName(), context == this ? 120 : 100);
 }
 
@@ -776,7 +903,7 @@ bool FolderNode::showWhenEmpty() const
 
   \sa ProjectExplorer::FileNode, ProjectExplorer::ProjectNode
 */
-VirtualFolderNode::VirtualFolderNode(const Utils::FilePath &folderPath) :
+VirtualFolderNode::VirtualFolderNode(const FilePath &folderPath) :
     FolderNode(folderPath)
 {
 }
@@ -794,7 +921,7 @@ VirtualFolderNode::VirtualFolderNode(const Utils::FilePath &folderPath) :
 /*!
   Creates an uninitialized project node object.
   */
-ProjectNode::ProjectNode(const Utils::FilePath &projectFilePath) :
+ProjectNode::ProjectNode(const FilePath &projectFilePath) :
     FolderNode(projectFilePath)
 {
     setPriority(DefaultProjectPriority);
@@ -802,13 +929,13 @@ ProjectNode::ProjectNode(const Utils::FilePath &projectFilePath) :
     setDisplayName(projectFilePath.fileName());
 }
 
-bool ProjectNode::canAddSubProject(const QString &proFilePath) const
+bool ProjectNode::canAddSubProject(const FilePath &proFilePath) const
 {
     Q_UNUSED(proFilePath)
     return false;
 }
 
-bool ProjectNode::addSubProject(const QString &proFilePath)
+bool ProjectNode::addSubProject(const FilePath &proFilePath)
 {
     Q_UNUSED(proFilePath)
     return false;
@@ -816,57 +943,70 @@ bool ProjectNode::addSubProject(const QString &proFilePath)
 
 QStringList ProjectNode::subProjectFileNamePatterns() const
 {
-    return QStringList();
+    return {};
 }
 
-bool ProjectNode::removeSubProject(const QString &proFilePath)
+bool ProjectNode::removeSubProject(const FilePath &proFilePath)
 {
     Q_UNUSED(proFilePath)
     return false;
 }
 
-bool ProjectNode::addFiles(const QStringList &filePaths, QStringList *notAdded)
+bool ProjectNode::addFiles(const FilePaths &filePaths, FilePaths *notAdded)
 {
-    Q_UNUSED(filePaths)
-    Q_UNUSED(notAdded)
+    if (BuildSystem *bs = buildSystem())
+        return bs->addFiles(this, filePaths, notAdded);
     return false;
 }
 
-bool ProjectNode::removeFiles(const QStringList &filePaths, QStringList *notRemoved)
+RemovedFilesFromProject ProjectNode::removeFiles(const FilePaths &filePaths, FilePaths *notRemoved)
 {
-    Q_UNUSED(filePaths)
-    Q_UNUSED(notRemoved)
+    if (BuildSystem *bs = buildSystem())
+        return bs->removeFiles(this, filePaths, notRemoved);
+    return RemovedFilesFromProject::Error;
+}
+
+bool ProjectNode::deleteFiles(const FilePaths &filePaths)
+{
+    if (BuildSystem *bs = buildSystem())
+        return bs->deleteFiles(this, filePaths);
     return false;
 }
 
-bool ProjectNode::deleteFiles(const QStringList &filePaths)
+bool ProjectNode::canRenameFile(
+    const Utils::FilePath &oldFilePath, const Utils::FilePath &newFilePath)
 {
-    Q_UNUSED(filePaths)
+    if (BuildSystem * const bs = buildSystem())
+        return bs->canRenameFile(this, oldFilePath, newFilePath);
     return false;
 }
 
-bool ProjectNode::canRenameFile(const QString &filePath, const QString &newFilePath)
+bool ProjectNode::renameFiles(const FilePairs &filesToRename, FilePaths *notRenamed)
 {
-    Q_UNUSED(filePath);
-    Q_UNUSED(newFilePath);
-    return true;
-}
-
-bool ProjectNode::renameFile(const QString &filePath, const QString &newFilePath)
-{
-    Q_UNUSED(filePath)
-    Q_UNUSED(newFilePath)
+    if (BuildSystem *bs = buildSystem())
+        return bs->renameFiles(this, filesToRename, notRenamed);
+    if (notRenamed)
+        *notRenamed = firstPaths(filesToRename);
     return false;
 }
 
-bool ProjectNode::supportsAction(ProjectAction, const Node *) const
+bool ProjectNode::addDependencies(const QStringList &dependencies)
 {
+    if (BuildSystem *bs = buildSystem())
+        return bs->addDependencies(this, dependencies);
     return false;
 }
 
-bool ProjectNode::deploysFolder(const QString &folder) const
+bool ProjectNode::supportsAction(ProjectAction action, const Node *node) const
 {
-    Q_UNUSED(folder);
+    if (BuildSystem *bs = buildSystem())
+        return bs->supportsAction(const_cast<ProjectNode *>(this), action, node);
+    return false;
+}
+
+bool ProjectNode::deploysFolder(const FilePath &folder) const
+{
+    Q_UNUSED(folder)
     return false;
 }
 
@@ -880,19 +1020,28 @@ ProjectNode *ProjectNode::projectNode(const Utils::FilePath &file) const
     return nullptr;
 }
 
-QVariant ProjectNode::data(Core::Id role) const
+QVariant ProjectNode::data(Utils::Id role) const
 {
-    Q_UNUSED(role);
-    return QVariant();
+    return m_fallbackData.value(role);
 }
 
-bool ProjectNode::setData(Core::Id role, const QVariant &value) const
+bool ProjectNode::setData(Utils::Id role, const QVariant &value) const
 {
-    Q_UNUSED(role);
-    Q_UNUSED(value);
+    Q_UNUSED(role)
+    Q_UNUSED(value)
     return false;
 }
 
+void ProjectNode::setFallbackData(Utils::Id key, const QVariant &value)
+{
+    m_fallbackData.insert(key, value);
+}
+
+BuildSystem *ProjectNode::buildSystem() const
+{
+    return activeBuildSystem(getProject());
+}
+
 bool FolderNode::isEmpty() const
 {
     return m_nodes.size() == 0;
@@ -916,10 +1065,10 @@ ContainerNode::ContainerNode(Project *project)
 
 QString ContainerNode::displayName() const
 {
-    QString name = m_project->displayName();
+    QString name = rawDisplayName();
 
-    const QFileInfo fi = m_project->projectFilePath().toFileInfo();
-    const QString dir = fi.isDir() ? fi.absoluteFilePath() : fi.absolutePath();
+    const FilePath fp = m_project->projectFilePath();
+    const FilePath dir = fp.isDir() ? fp.absoluteFilePath() : fp.absolutePath();
     if (Core::IVersionControl *vc = Core::VcsManager::findVersionControlForDirectory(dir)) {
         QString vcsTopic = vc->vcsTopic(dir);
         if (!vcsTopic.isEmpty())
@@ -929,6 +1078,11 @@ QString ContainerNode::displayName() const
     return name;
 }
 
+QString ContainerNode::rawDisplayName() const
+{
+    return m_project->displayName();
+}
+
 bool ContainerNode::supportsAction(ProjectAction action, const Node *node) const
 {
     const Node *rootNode = m_project->rootProjectNode();
@@ -950,4 +1104,58 @@ void ContainerNode::handleSubTreeChanged(FolderNode *node)
     m_project->handleSubTreeChanged(node);
 }
 
+/*!
+    \class ProjectExplorer::DirectoryIcon
+
+    The DirectoryIcon class represents a directory icon with an overlay.
+
+    The QIcon is created on demand and globally cached, so other DirectoryIcon
+    instances with the same overlay share the same QIcon instance.
+*/
+
+/*!
+    Creates a DirectoryIcon for the specified \a overlay.
+*/
+DirectoryIcon::DirectoryIcon(const QString &overlay)
+    : m_overlay(overlay)
+{}
+
+/*!
+    Returns the icon for this DirectoryIcon. Calling this method is only safe in the UI thread.
+*/
+QIcon DirectoryIcon::icon() const
+{
+    QTC_CHECK(isMainThread());
+    const auto it = m_cache.find(m_overlay);
+    if (it != m_cache.end())
+        return it.value();
+    const QIcon icon = Utils::FileIconProvider::directoryIcon(m_overlay);
+    m_cache.insert(m_overlay, icon);
+    return icon;
+}
+
+ResourceFileNode::ResourceFileNode(const FilePath &filePath, const QString &qrcPath, const QString &displayName)
+    : FileNode(filePath, FileNode::fileTypeForFileName(filePath))
+    , m_qrcPath(qrcPath)
+    , m_displayName(displayName)
+{
+}
+
+QString ResourceFileNode::displayName() const
+{
+    return m_displayName;
+}
+
+QString ResourceFileNode::qrcPath() const
+{
+    return m_qrcPath;
+}
+
+bool ResourceFileNode::supportsAction(ProjectAction action, const Node *node) const
+{
+    if (action == HidePathActions)
+        return false;
+    return parentFolderNode()->supportsAction(action, node);
+}
+
 } // namespace ProjectExplorer
