@@ -3,11 +3,13 @@
 
 #pragma once
 
+#include <projectexplorer/kitaspect.h>
 #include <projectexplorer/runconfigurationaspects.h>
 
 #include <solutions/tasking/tasktreerunner.h>
 
 #include <utils/filepath.h>
+#include <utils/listmodel.h>
 
 namespace Python::Internal {
 
@@ -25,6 +27,12 @@ public:
     static Interpreter defaultInterpreter();
     static Interpreter interpreter(const QString &interpreterId);
     static void setInterpreter(const QList<Interpreter> &interpreters, const QString &defaultId);
+    static Interpreter createInterpreter(
+        const Utils::FilePath &python,
+        const QString &defaultName,
+        const QString &suffix = {},
+        const ProjectExplorer::DetectionSource &detectionSource = {});
+
     static void addInterpreter(const Interpreter &interpreter, bool isDefault = false);
     static Interpreter addInterpreter(const Utils::FilePath &interpreterPath,
                                       bool isDefault = false,
@@ -45,6 +53,17 @@ public:
     static QList<Interpreter> detectPythonVenvs(const Utils::FilePath &path);
     static void addKitsForInterpreter(const Interpreter &interpreter, bool force);
     static void removeKitsForInterpreter(const Interpreter &interpreter);
+    static bool interpreterIsValid(const Interpreter &interpreter);
+
+    static std::optional<Tasking::ExecutableItem> autoDetect(
+        ProjectExplorer::Kit *kit,
+        const Utils::FilePaths &searchPaths,
+        const ProjectExplorer::DetectionSource &detectionSource,
+        const ProjectExplorer::LogCallback &logCallback);
+    static void removeDetectedPython(
+        const QString &detectionSource, const ProjectExplorer::LogCallback &logCallback);
+    static void listDetectedPython(
+        const QString &detectionSource, const ProjectExplorer::LogCallback &logCallback);
 
 signals:
     void interpretersChanged(const QList<Interpreter> &interpreters, const QString &defaultId);
@@ -52,14 +71,6 @@ signals:
     void pylsEnabledChanged(const bool enabled);
     void virtualEnvironmentCreated(const Utils::FilePath &venvPython);
 
-public slots:
-    void detectPythonOnDevice(const Utils::FilePaths &searchPaths,
-                              const QString &deviceName,
-                              const QString &detectionSource,
-                              QString *logMessage);
-    void removeDetectedPython(const QString &detectionSource, QString *logMessage);
-    void listDetectedPython(const QString &detectionSource, QString *logMessage);
-
 private:
     void disableOutdatedPyls();
     void disableOutdatedPylsNow();
@@ -71,11 +82,13 @@ private:
     QString m_defaultInterpreterId;
     bool m_pylsEnabled = true;
     QString m_pylsConfiguration;
-    Tasking::TaskTreeRunner m_taskTreeRunner;
+    Tasking::SingleTaskTreeRunner m_taskTreeRunner;
 
     static void saveSettings();
 };
 
-void setupPythonSettings(QObject *guard);
+void setupPythonSettings();
+
+Utils::ListModel<ProjectExplorer::Interpreter> *createInterpreterModel(QObject *parent);
 
 } // Python::Internal
