@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 #pragma once
 
 #include "abstractformeditortool.h"
@@ -29,7 +7,6 @@
 #include "selectionindicator.h"
 
 #include <QObject>
-#include <QScopedPointer>
 #include <QPointer>
 
 namespace QmlDesigner {
@@ -40,7 +17,7 @@ class DragTool : public AbstractFormEditorTool
 {
 public:
     DragTool(FormEditorView *editorView);
-    virtual ~DragTool();
+    ~DragTool() override;
 
     void mousePressEvent(const QList<QGraphicsItem*> &itemList, QGraphicsSceneMouseEvent *event) override;
     void mouseMoveEvent(const QList<QGraphicsItem*> &itemList, QGraphicsSceneMouseEvent *event) override;
@@ -75,27 +52,32 @@ public:
 
     void clearMoveDelay();
 
+    void focusLost() override;
+
 protected:
     void abort();
     void createQmlItemNode(const ItemLibraryEntry &itemLibraryEntry, const QmlItemNode &parentNode, const QPointF &scenePos);
-    void createQmlItemNodeFromImage(const QString &imageName, const QmlItemNode &parentNode, const QPointF &scenePos);
-    FormEditorItem *targetContainerOrRootItem(const QList<QGraphicsItem*> &itemList, FormEditorItem *urrentItem = 0);
+    void createQmlItemNodeFromImage(const QString &imagePath, const QmlItemNode &parentNode, const QPointF &scenePos);
+    void createQmlItemNodeFromFont(const QString &fontPath, const QmlItemNode &parentNode, const QPointF &scenePos);
+    FormEditorItem *targetContainerOrRootItem(const QList<QGraphicsItem *> &itemList,
+                                              const QList<FormEditorItem *> &currentItems = {});
     void begin(QPointF scenePos);
     void end();
     void end(Snapper::Snapping useSnapping);
     void move(const QPointF &scenePos, const QList<QGraphicsItem *> &itemList);
-    void createDragNode(const QMimeData *mimeData, const QPointF &scenePosition, const QList<QGraphicsItem *> &itemList);
+    void createDragNodes(const QMimeData *mimeData, const QPointF &scenePosition, const QList<QGraphicsItem *> &itemList);
     void commitTransaction();
+    void handleView3dDrop();
 
 private:
     MoveManipulator m_moveManipulator;
     SelectionIndicator m_selectionIndicator;
-    QPointer<FormEditorItem> m_movingItem;
+    QList<FormEditorItem *> m_movingItems;
     RewriterTransaction m_rewriterTransaction;
-    QmlItemNode m_dragNode;
+    QList<QmlItemNode> m_dragNodes;
     bool m_blockMove;
     QPointF m_startPoint;
-    bool m_isAborted;
+    bool m_isAborted = false;
 };
 
 } // namespace QmlDesigner
