@@ -1,37 +1,18 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "documentmanager.h"
 #include "qmldesignerplugin.h"
 
+#include <auxiliarydataproperties.h>
+#include <bindingproperty.h>
+#include <modelutils.h>
 #include <modelnode.h>
-#include <qmlitemnode.h>
+#include <nodelistproperty.h>
 #include <nodemetainfo.h>
 #include <nodeproperty.h>
-#include <nodelistproperty.h>
-#include <bindingproperty.h>
+#include <qmldesignerprojectmanager.h>
+#include <qmlitemnode.h>
 #include <variantproperty.h>
 
 #include <utils/qtcassert.h>
@@ -42,28 +23,31 @@
 #include <coreplugin/vcsmanager.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/messagebox.h>
-#include <projectexplorer/projectnodes.h>
+
 #include <projectexplorer/project.h>
+#include <projectexplorer/projectmanager.h>
+#include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projecttree.h>
-#include <projectexplorer/session.h>
 
 #include <qmakeprojectmanager/qmakenodes.h>
 #include <qmakeprojectmanager/qmakeproject.h>
 
 #include <QMessageBox>
 
+using namespace ProjectExplorer;
 using namespace Utils;
 
 namespace QmlDesigner {
 
 Q_LOGGING_CATEGORY(documentManagerLog, "qtc.qtquickdesigner.documentmanager", QtWarningMsg)
 
-static inline QmlDesigner::DesignDocument* designDocument()
+namespace {
+QmlDesigner::DesignDocument *designDocument()
 {
     return QmlDesigner::QmlDesignerPlugin::instance()->documentManager().currentDesignDocument();
 }
 
-static inline QHash<PropertyName, QVariant> getProperties(const ModelNode &node)
+QHash<PropertyName, QVariant> getProperties(const ModelNode &node)
 {
     QHash<PropertyName, QVariant> propertyHash;
     if (QmlObjectNode::isValidQmlObjectNode(node)) {
@@ -72,7 +56,8 @@ static inline QHash<PropertyName, QVariant> getProperties(const ModelNode &node)
             if (abstractProperty.isVariantProperty()
                     || (abstractProperty.isBindingProperty()
                         && !abstractProperty.name().contains("anchors.")))
-                propertyHash.insert(abstractProperty.name(), QmlObjectNode(node).instanceValue(abstractProperty.name()));
+                propertyHash.insert(abstractProperty.name().toByteArray(),
+                                    QmlObjectNode(node).instanceValue(abstractProperty.name()));
         }
 
         if (QmlItemNode::isValidQmlItemNode(node)) {
@@ -90,27 +75,28 @@ static inline QHash<PropertyName, QVariant> getProperties(const ModelNode &node)
     return propertyHash;
 }
 
-static inline void applyProperties(ModelNode &node, const QHash<PropertyName, QVariant> &propertyHash)
+void applyProperties(ModelNode &node, const QHash<PropertyName, QVariant> &propertyHash)
 {
-    const QHash<PropertyName, QVariant> auxiliaryData  = node.auxiliaryData();
+    const auto auxiliaryData = node.auxiliaryData(AuxiliaryDataType::NodeInstancePropertyOverwrite);
+    for (const auto &element : auxiliaryData)
+        node.removeAuxiliaryData(AuxiliaryDataType::NodeInstancePropertyOverwrite, element.first);
 
-    const QList<PropertyName> propertyNames = auxiliaryData.keys();
-    for (const PropertyName &propertyName : propertyNames) {
-        if (node.hasAuxiliaryData(propertyName))
-            node.setAuxiliaryData(propertyName, QVariant());
-    }
-
-    for (auto propertyIterator = propertyHash.cbegin(), end = propertyHash.cend();
-              propertyIterator != end;
-              ++propertyIterator) {
-        const PropertyName propertyName = propertyIterator.key();
-        if (propertyName == "width" || propertyName == "height") {
-            node.setAuxiliaryData(propertyIterator.key(), propertyIterator.value());
+    auto needsOverwrite = [&node](const auto& check, const auto& name, const auto& instanceValue) {
+        if (check == name) {
+            VariantProperty property = node.variantProperty(name);
+            if (property.isValid() && property.value() != instanceValue)
+                return true;
         }
+        return false;
+    };
+
+    for (const auto& [name, value] : propertyHash.asKeyValueRange()) {
+        if (needsOverwrite("width", name, value) || needsOverwrite("height", name, value))
+            node.setAuxiliaryData(AuxiliaryDataType::NodeInstancePropertyOverwrite, name, value);
     }
 }
 
-static void openFileComponentForFile(const QString &fileName)
+void openFileComponentForFile(const QString &fileName)
 {
     QmlDesignerPlugin::instance()->viewManager().nextFileIsCalledInternally();
     Core::EditorManager::openEditor(FilePath::fromString(fileName),
@@ -118,17 +104,17 @@ static void openFileComponentForFile(const QString &fileName)
                                     Core::EditorManager::DoNotMakeVisible);
 }
 
-static void openFileComponent(const ModelNode &modelNode)
+void openFileComponent(const ModelNode &modelNode)
 {
-    openFileComponentForFile(modelNode.metaInfo().componentFileName());
+    openFileComponentForFile(ModelUtils::componentFilePath(modelNode));
 }
 
-static void openFileComponentForDelegate(const ModelNode &modelNode)
+void openFileComponentForDelegate(const ModelNode &modelNode)
 {
     openFileComponent(modelNode.nodeProperty("delegate").modelNode());
 }
 
-static void openComponentSourcePropertyOfLoader(const ModelNode &modelNode)
+void openComponentSourcePropertyOfLoader(const ModelNode &modelNode)
 {
     QmlDesignerPlugin::instance()->viewManager().nextFileIsCalledInternally();
 
@@ -147,12 +133,12 @@ static void openComponentSourcePropertyOfLoader(const ModelNode &modelNode)
     }
 
     Core::EditorManager::openEditor(FilePath::fromString(
-                                        componentModelNode.metaInfo().componentFileName()),
+                                        ModelUtils::componentFilePath(componentModelNode)),
                                     Utils::Id(),
                                     Core::EditorManager::DoNotMakeVisible);
 }
 
-static void openSourcePropertyOfLoader(const ModelNode &modelNode)
+void openSourcePropertyOfLoader(const ModelNode &modelNode)
 {
     QmlDesignerPlugin::instance()->viewManager().nextFileIsCalledInternally();
 
@@ -165,14 +151,13 @@ static void openSourcePropertyOfLoader(const ModelNode &modelNode)
                                     Core::EditorManager::DoNotMakeVisible);
 }
 
-
-static void handleComponent(const ModelNode &modelNode)
+void handleComponent(const ModelNode &modelNode)
 {
     if (modelNode.nodeSourceType() == ModelNode::NodeWithComponentSource)
         designDocument()->changeToSubComponent(modelNode);
 }
 
-static void handleDelegate(const ModelNode &modelNode)
+void handleDelegate(const ModelNode &modelNode)
 {
     if (modelNode.metaInfo().isView()
             && modelNode.hasNodeProperty("delegate")
@@ -180,7 +165,7 @@ static void handleDelegate(const ModelNode &modelNode)
         designDocument()->changeToSubComponent(modelNode.nodeProperty("delegate").modelNode());
 }
 
-static void handleTabComponent(const ModelNode &modelNode)
+void handleTabComponent(const ModelNode &modelNode)
 {
     if (modelNode.hasNodeProperty("component")
             && modelNode.nodeProperty("component").modelNode().nodeSourceType() == ModelNode::NodeWithComponentSource) {
@@ -188,7 +173,7 @@ static void handleTabComponent(const ModelNode &modelNode)
     }
 }
 
-static inline void openInlineComponent(const ModelNode &modelNode)
+void openInlineComponent(const ModelNode &modelNode)
 {
     if (!modelNode.metaInfo().isValid())
         return;
@@ -198,7 +183,7 @@ static inline void openInlineComponent(const ModelNode &modelNode)
     handleTabComponent(modelNode);
 }
 
-static bool isFileComponent(const ModelNode &node)
+bool isFileComponent(const ModelNode &node)
 {
     if (node.isValid()
             && node.metaInfo().isValid()
@@ -208,7 +193,7 @@ static bool isFileComponent(const ModelNode &node)
     return false;
 }
 
-static bool hasDelegateWithFileComponent(const ModelNode &node)
+bool hasDelegateWithFileComponent(const ModelNode &node)
 {
     if (node.isValid()
             && node.metaInfo().isValid()
@@ -220,12 +205,9 @@ static bool hasDelegateWithFileComponent(const ModelNode &node)
     return false;
 }
 
-static bool isLoaderWithSourceComponent(const ModelNode &modelNode)
+bool isLoaderWithSourceComponent(const ModelNode &modelNode)
 {
-    if (modelNode.isValid()
-            && modelNode.metaInfo().isValid()
-            && modelNode.metaInfo().isSubclassOf("QtQuick.Loader")) {
-
+    if (modelNode.isValid() && modelNode.metaInfo().isQtQuickLoader()) {
         if (modelNode.hasNodeProperty("sourceComponent"))
             return true;
         if (modelNode.hasNodeListProperty("component"))
@@ -235,37 +217,32 @@ static bool isLoaderWithSourceComponent(const ModelNode &modelNode)
     return false;
 }
 
-static bool hasSourceWithFileComponent(const ModelNode &modelNode)
+bool hasSourceWithFileComponent(const ModelNode &modelNode)
 {
-    if (modelNode.isValid()
-            && modelNode.metaInfo().isValid()
-            && modelNode.metaInfo().isSubclassOf("QtQuick.Loader")
-            && modelNode.hasVariantProperty("source"))
+    if (modelNode.isValid() && modelNode.metaInfo().isQtQuickLoader()
+        && modelNode.hasVariantProperty("source"))
         return true;
 
     return false;
 }
 
-DocumentManager::DocumentManager()
-    : QObject()
-{
-}
-
-DocumentManager::~DocumentManager()
-{
-    qDeleteAll(m_designDocumentHash);
-}
+} // namespace
 
 void DocumentManager::setCurrentDesignDocument(Core::IEditor *editor)
 {
     if (editor) {
-        m_currentDesignDocument = m_designDocumentHash.value(editor);
-        if (m_currentDesignDocument == nullptr) {
-            m_currentDesignDocument = new DesignDocument;
-            m_designDocumentHash.insert(editor, m_currentDesignDocument);
+        auto found = m_designDocuments.find(editor);
+        if (found == m_designDocuments.end()) {
+            auto &inserted = m_designDocuments[editor] = std::make_unique<DesignDocument>(
+                editor->document()->filePath().toUrlishString(),
+                m_projectManager.projectStorageDependencies(),
+                m_externalDependencies);
+            m_currentDesignDocument = inserted.get();
             m_currentDesignDocument->setEditor(editor);
+        } else {
+            m_currentDesignDocument = found->second.get();
         }
-    } else if (!m_currentDesignDocument.isNull()) {
+    } else if (m_currentDesignDocument) {
         m_currentDesignDocument->resetToDocumentModel();
         m_currentDesignDocument.clear();
     }
@@ -284,11 +261,27 @@ bool DocumentManager::hasCurrentDesignDocument() const
 void DocumentManager::removeEditors(const QList<Core::IEditor *> &editors)
 {
     for (Core::IEditor *editor : editors)
-        delete m_designDocumentHash.take(editor).data();
+        m_designDocuments.erase(editor);
+}
+
+void DocumentManager::resetPossibleImports()
+{
+    for (const auto &[key, value] : m_designDocuments) {
+        if (RewriterView *view = value->rewriterView())
+            view->resetPossibleImports();
+    }
+}
+
+const GeneratedComponentUtils &DocumentManager::generatedComponentUtils() const
+{
+   return m_generatedComponentUtils;
 }
 
 bool DocumentManager::goIntoComponent(const ModelNode &modelNode)
 {
+    QImage image = QmlDesignerPlugin::instance()->viewManager().takeFormEditorScreenshot();
+    const QPoint offset = image.offset();
+    image.setOffset(offset - QmlItemNode(modelNode).instancePosition().toPoint());
     if (modelNode.isValid() && modelNode.isComponent() && designDocument()) {
         QmlDesignerPlugin::instance()->viewManager().setComponentNode(modelNode);
         QHash<PropertyName, QVariant> oldProperties = getProperties(modelNode);
@@ -306,6 +299,8 @@ bool DocumentManager::goIntoComponent(const ModelNode &modelNode)
         ModelNode rootModelNode = designDocument()->rewriterView()->rootModelNode();
         applyProperties(rootModelNode, oldProperties);
 
+        rootModelNode.setAuxiliaryData(contextImageProperty, image);
+
         return true;
     }
 
@@ -319,11 +314,10 @@ void DocumentManager::goIntoComponent(const QString &fileName)
 
 bool DocumentManager::createFile(const QString &filePath, const QString &contents)
 {
-    Utils::TextFileFormat textFileFormat;
-    textFileFormat.codec = Core::EditorManager::defaultTextCodec();
-    QString errorMessage;
+    TextFileFormat textFileFormat;
+    textFileFormat.setEncoding(Core::EditorManager::defaultTextEncoding());
 
-    return textFileFormat.writeFile(Utils::FilePath::fromString(filePath), contents, &errorMessage);
+    return textFileFormat.writeFile(FilePath::fromString(filePath), contents).has_value();
 }
 
 void DocumentManager::addFileToVersionControl(const QString &directoryPath, const QString &newFilePath)
@@ -356,42 +350,59 @@ Utils::FilePath DocumentManager::currentProjectDirPath()
 {
     QTC_ASSERT(QmlDesignerPlugin::instance(), return {});
 
-    if (!QmlDesignerPlugin::instance()->currentDesignDocument())
+    if (!QmlDesignerPlugin::instance()->currentDesignDocument()) {
+        if (ProjectExplorer::Project *project = ProjectExplorer::ProjectManager::startupProject())
+            return project->projectDirectory();
         return {};
+    }
 
     Utils::FilePath qmlFileName = QmlDesignerPlugin::instance()->currentDesignDocument()->fileName();
 
-    ProjectExplorer::Project *project = ProjectExplorer::SessionManager::projectForFile(qmlFileName);
+    ProjectExplorer::Project *project = ProjectExplorer::ProjectManager::projectForFile(qmlFileName);
+
     if (project)
         return project->projectDirectory();
 
-    const QList projects = ProjectExplorer::SessionManager::projects();
+    const QList projects = ProjectExplorer::ProjectManager::projects();
     for (auto p : projects) {
-        if (qmlFileName.startsWith(p->projectDirectory().toString()))
+        if (qmlFileName.startsWith(p->projectDirectory().toUrlishString()))
             return p->projectDirectory();
     }
 
     return {};
 }
 
+QString DocumentManager::currentProjectName()
+{
+    QTC_ASSERT(QmlDesignerPlugin::instance(), return {});
+
+    if (!QmlDesignerPlugin::instance()->currentDesignDocument())
+        return {};
+
+    Utils::FilePath qmlFileName = QmlDesignerPlugin::instance()->currentDesignDocument()->fileName();
+    ProjectExplorer::Project *project = ProjectExplorer::ProjectManager::projectForFile(qmlFileName);
+
+    return project ? project->displayName() : "";
+}
+
 QStringList DocumentManager::isoIconsQmakeVariableValue(const QString &proPath)
 {
     ProjectExplorer::Node *node = ProjectExplorer::ProjectTree::nodeForFile(Utils::FilePath::fromString(proPath));
     if (!node) {
         qCWarning(documentManagerLog) << "No node for .pro:" << proPath;
-        return QStringList();
+        return {};
     }
 
     ProjectExplorer::Node *parentNode = node->parentFolderNode();
     if (!parentNode) {
         qCWarning(documentManagerLog) << "No parent node for node at" << proPath;
-        return QStringList();
+        return {};
     }
 
     auto proNode = dynamic_cast<QmakeProjectManager::QmakeProFileNode*>(parentNode);
     if (!proNode) {
         qCWarning(documentManagerLog) << "Parent node for node at" << proPath << "could not be converted to a QmakeProFileNode";
-        return QStringList();
+        return {};
     }
 
     return proNode->variableValue(QmakeProjectManager::Variable::IsoIcons);
@@ -428,7 +439,7 @@ void DocumentManager::findPathToIsoProFile(bool *iconResourceFileAlreadyExists,
     QString *resourceFileProPath, const QString &isoIconsQrcFile)
 {
     Utils::FilePath qmlFileName = QmlDesignerPlugin::instance()->currentDesignDocument()->fileName();
-    ProjectExplorer::Project *project = ProjectExplorer::SessionManager::projectForFile(qmlFileName);
+    ProjectExplorer::Project *project = ProjectExplorer::ProjectManager::projectForFile(qmlFileName);
     ProjectExplorer::Node *node = ProjectExplorer::ProjectTree::nodeForFile(qmlFileName)->parentFolderNode();
     ProjectExplorer::Node *iconQrcFileNode = nullptr;
 
@@ -438,8 +449,10 @@ void DocumentManager::findPathToIsoProFile(bool *iconResourceFileAlreadyExists,
         if (node->isVirtualFolderType() && node->displayName() == "Resources") {
             ProjectExplorer::FolderNode *virtualFolderNode = node->asFolderNode();
             if (QTC_GUARD(virtualFolderNode)) {
-                for (int subFolderIndex = 0; subFolderIndex < virtualFolderNode->folderNodes().size() && !iconQrcFileNode; ++subFolderIndex) {
-                    ProjectExplorer::FolderNode *subFolderNode = virtualFolderNode->folderNodes().at(subFolderIndex);
+                QList<FolderNode *> folderNodes;
+                virtualFolderNode->forEachFolderNode([&](FolderNode *fn) { folderNodes.append(fn); });
+                for (int subFolderIndex = 0; subFolderIndex < folderNodes.size() && !iconQrcFileNode; ++subFolderIndex) {
+                    ProjectExplorer::FolderNode *subFolderNode = folderNodes.at(subFolderIndex);
 
                     qCDebug(documentManagerLog) << "Checking if" << subFolderNode->displayName() << "("
                         << subFolderNode << ") is" << isoIconsQrcFile;
@@ -448,7 +461,7 @@ void DocumentManager::findPathToIsoProFile(bool *iconResourceFileAlreadyExists,
                         qCDebug(documentManagerLog) << "Found" << isoIconsQrcFile << "in" << virtualFolderNode->filePath();
 
                         iconQrcFileNode = subFolderNode;
-                        *resourceFileProPath = iconQrcFileNode->parentProjectNode()->filePath().toString();
+                        *resourceFileProPath = iconQrcFileNode->parentProjectNode()->filePath().toUrlishString();
                     }
                 }
             }
@@ -464,14 +477,14 @@ void DocumentManager::findPathToIsoProFile(bool *iconResourceFileAlreadyExists,
     if (!iconQrcFileNode) {
         // The QRC file that we want doesn't exist or is not listed under RESOURCES in the .pro.
         if (project)
-            *resourceFilePath = project->projectDirectory().toString() + "/" + isoIconsQrcFile;
+            *resourceFilePath = project->projectDirectory().toUrlishString() + "/" + isoIconsQrcFile;
 
         // We assume that the .pro containing the QML file is an acceptable place to add the .qrc file.
         ProjectExplorer::ProjectNode *projectNode = ProjectExplorer::ProjectTree::nodeForFile(qmlFileName)->parentProjectNode();
-        *resourceFileProPath = projectNode->filePath().toString();
+        *resourceFileProPath = projectNode->filePath().toUrlishString();
     } else {
         // We found the QRC file that we want.
-        QString projectDirectory = ProjectExplorer::ProjectTree::projectForNode(iconQrcFileNode)->projectDirectory().toString();
+        QString projectDirectory = ProjectExplorer::ProjectTree::projectForNode(iconQrcFileNode)->projectDirectory().toUrlishString();
         *resourceFilePath = projectDirectory + "/" + isoIconsQrcFile;
     }
 
@@ -518,7 +531,7 @@ bool DocumentManager::belongsToQmakeProject()
         return false;
 
     Utils::FilePath qmlFileName = QmlDesignerPlugin::instance()->currentDesignDocument()->fileName();
-    ProjectExplorer::Project *project = ProjectExplorer::SessionManager::projectForFile(qmlFileName);
+    ProjectExplorer::Project *project = ProjectExplorer::ProjectManager::projectForFile(qmlFileName);
     if (!project)
         return false;
 
@@ -538,6 +551,13 @@ Utils::FilePath DocumentManager::currentResourcePath()
     if (contentFilePath.exists())
         return contentFilePath;
 
+    const auto project = ProjectManager::startupProject();
+    const QString baseName = project->rootProjectDirectory().baseName() + "Content";
+
+    contentFilePath = resourcePath.pathAppended(baseName);
+    if (contentFilePath.exists())
+        return contentFilePath;
+
     return resourcePath;
 }
 
